/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __maya_MayaBrush_h__
#define __maya_MayaBrush_h__

namespace fe
{

class MayaNode;

/**************************************************************************//**
    @brief Maya Tool

	Maya's parallel to the Houdini brush is a MPxContext tool.
	The MayaContext singleton class provided by FE is not related.

	@ingroup maya
*//***************************************************************************/
class MayaBrush:
	public MPxContext,
	public MetaBrush
{
	public:

	class Command: public MPxContextCommand
	{
		public:
						Command(void)										{}
	virtual	MPxContext*	makeObj(void)	{ return new MayaBrush(); }
	static	void*		creator(void)	{ return new MayaBrush::Command(); }
	};

							MayaBrush(void);
virtual						~MayaBrush(void);

virtual	void				toolOnSetup(MEvent &event);
virtual	void				toolOffCleanup();
virtual	MStatus				doEnterRegion(MEvent &event);
virtual	MStatus				doPress(MEvent &event);
virtual	MStatus				doRelease(MEvent &event);
virtual	MStatus				doDrag(MEvent &event);
		void				doMove(I32 a_mouseX,I32 a_mouseY);
virtual	void				deleteAction(void);
virtual	void				completeAction(void);
virtual	void				abortAction(void);
virtual	bool				processNumericalInput(const MDoubleArray& rValues,
									const MIntArray& rFlags,bool isAbsolute);

static	void				staticSetup(void);
static	void				idleAdjust(void);

static	MayaBrush*			ms_pMayaBrush;
static	MayaNode*			ms_pCurrentNode;
static	BWORD				ms_mayaBrushCurrent;
static	MCallbackId			ms_idleCallback;


	private:
		void				interpretMouse(MEvent& mayaEvent,
									WindowEvent::State state);
		void				brush(void);

		Matrix<4,4,Real>	m_projection;

		I32					m_mouseX;
		I32					m_mouseY;
};

} // namespace

#endif /* __maya_MayaBrush_h__ */

