/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */


#ifndef __platform_Exception_h__
#define __platform_Exception_h__

// TODO: the use of Statics here is not safe.  Try to find a better way.

#if FE_CODEGEN>FE_DEBUG
#define feX throw fe::Exception
// funkiness to get __FILE__ and __LINE__ with varargs
#elif FE_COMPILER==FE_GNU
#define feX fe::Exception::stage(__FILE__,__LINE__,__PRETTY_FUNCTION__), throw fe::Exception
#elif FE_COMPILER==FE_MICROSOFT
#define feX fe::Exception::stage(__FILE__,__LINE__,__FUNCTION__), throw fe::Exception
#else
#define feX fe::Exception::stage(__FILE__,__LINE__,__func__), throw fe::Exception
#endif

#define feAssert(cond,msg) {if(!cond){feX(msg);}}

namespace fe
{

/**************************************************************************//**
	@brief Generic exception carrying a fe::String payload

	@ingroup platform
*//***************************************************************************/
class FE_DL_PUBLIC Exception
{
	public:
						Exception(void);
						Exception(const char *fmt, ...);
						Exception(const char *location, const char *fmt, ...);
						Exception(const Result &result, const char *fmt, ...);
						Exception(const Result &result, const char *location,
									const char *fmt, ...);
						Exception(const Result &result, const char *location,
									String fmt, ...);
						Exception(const Result &result);
						Exception(const Exception &other);

virtual					~Exception(void);

		Exception		&operator=(const Exception &other);

						/// @brief Return the fe::String payload
const	String			&getMessage(void) const		{ return m_message; }
const	String			&getLocation(void) const	{ return m_location; }
const	String			&getFile(void) const		{ return m_file; }
const	String			&getFunction(void) const	{ return m_function; }
		Result			getResult(void) const		{ return m_result; }
		int				getLine(void) const			{ return m_line; }

						/// @brief Send the message to the global log
		void			log(void);

						/** @brief Assert instead of except,
							if selected

							If the env variable "FE_E_ASSERT" is set,
							exceptions cause an assertion when they create.
							They don't have to be thrown first.

							@internal */
virtual	void			checkAssert(void);
						/**	Not thread safe, so exceptions thrown from
							multiple threads are in a race condition on
							file and line.
							*/
static	void			stage(const char *file, int line, const char* function);

	protected:
		String	m_message;
		String	m_location;
		Result	m_result;
		String	m_file;
		int		m_line;
		String	m_function;
static	String	ms_file;
static	int		ms_line;
static	String	ms_function;

};

FE_DL_PUBLIC String print(const Exception& a_rException);

} /* namespace */

#endif /* __platform_Exception_h__ */
